#!/usr/bin/perl

# Program latlong.pl

# This program converts latitude and longitude values between three different formats:

# Degree, minute second (DMS) -- e.g., 179o82'45"
# GPS (DM.M) -- e.g., 14o45.667'
# Decimal (D.DDDD) -- e.g., 14.78234o

# (c) Gary C. Kessler, 2013 [gck@garykessler.net]


#use Encode qw/encode_utf8 decode_utf8/;
use utf8;
binmode STDOUT, ":utf8";

# Initialize global variables
$version = "1.0";
$build_date = "19 September 2013";

print "\nLatitude/Longitude Converter V$version - Gary C. Kessler ($build_date)\n\n";

print "What is the format for input? Enter:\n";
print "   D for DMS format (e.g., DDD", chr(176), "MM'SS\")\n";
print "   G for GPS format (e.g., DDD", chr(176), "MMM.MMM')\n";
print "   N for decimal format (e.g., DDD.DDD", chr(176), ")\n";

print "\nEnter format (D, G, or N): ";
chomp ($format = <STDIN>);
$format = uc ($format);

if ($format ne "D" && $format ne "N" && $format ne "G")
  {
  print "\nInvalid format type\n\n";
  exit;
  }

# Process each format similarly but different....

CASE:
  {
  $format eq "D" && do
    {
    DMS_START:
    do
      {
      print "\nProvide DMS coordinates --\n";
      print "  Enter degrees (0-359; 999 to stop): ";
      chomp ($DD = <STDIN>);
      if ($DD == 999)
        { last CASE; };
      print "  Enter minutes (0-59): ";
      chomp ($MM = <STDIN>);
      print "  Enter seconds (0-59): ";
      chomp ($SS = <STDIN>);

# Validate input -- D = 0-359, M/S = 0-59

      if ($DD < 0 || $DD >= 360 || $MM < 0 || $MM >= 60 ||
          $SS < 0 || $SS >= 60)
          {
          print "*** Invalid entry ***\n";
          $ok = 0;
          }
        else
          { $ok = 1; } 
      }
      until ($ok == 1);

    $GD = $DD;
    $GM = $MM + ($SS/60);

    $N = $DD + (($MM*60) + $SS) / 3600;

    printf "\n\nDMS coordinates: %03d%s%02d\"%02d'\n", $DD,chr(176),$MM,$SS;
    printf "            GPS: %03d%s%08.5f\"\n", $GD,chr(176),$GM;
    printf "        Decimal: %09.5f%s\n", $N,chr(176);
    goto DMS_START;
    };

  $format eq "G" && do
    {
    GPS_START:
    do
      {
      print "\nProvide GPS coordinates --\n";
      print "  Enter degrees (0-359; 999 to stop): ";
      chomp ($GD = <STDIN>);
      if ($GD == 999)
        { last CASE; };
      print "  Enter minutes (0-59.99999): ";
      chomp ($GM = <STDIN>);

# Validate input -- GD = 0-359, GM = [0-59)

      if ($GD < 0 || $GD >= 360 || $GM < 0 || $GM >= 60)
          {
          print "*** Invalid entry ***\n";
          $ok = 0;
          }
        else
          { $ok = 1; } 
      }
      until ($ok == 1);

    $DD = $GD;
    $MM = int ($GM);
    $SS = ($GM-$MM)*60;

    $N = $GD + ($GM/60);

    printf "\n\nGPS coordinates: %03d%s%08.5f\"\n", $GD,chr(176),$GM;
    printf "            DMS: %03d%s%02d\"%02d'\n", $DD,chr(176),$MM,$SS;
    printf "        Decimal: %09.5f%s\n", $N,chr(176);
    goto GPS_START;
    };

  $format eq "N" && do
    {
    DEC_START:
    do
      {
      print "\nProvide decimal coordinates --\n";
      print "  Enter degrees (0-359.99999; 999 to stop): ";
      chomp ($N = <STDIN>);
      if ($N == 999)
        { last CASE; };

# Validate input -- N = [0-359)

      if ($N < 0 || $N >= 360)
          {
          print "*** Invalid entry ***\n";
          $ok = 0;
          }
        else
          { $ok = 1; } 
      }
      until ($ok == 1);

    $GD = int ($N);
    $GM = ($N-$GD)*60;

    $DD = int ($N);
    $MM = int ($GM);
    $SS = ($GM-$MM)*60;

    printf "\n\n Decimal coordinates: %09.5f%s\n", $N,chr(176);
    printf "                 DMS: %03d%s%02d\"%02d'\n", $DD,chr(176),$MM,$SS;
    printf "                 GPS: %03d%s%08.5f\"\n", $GD,chr(176),$GM;
    goto DEC_START;
    };

  }

print "\nDone!\n\n";

